@@ -22,20 +22,20 @@ import pygtrie
 class _TrieFactoryParameteriser(object):
     # pylint: disable=no-self-argument, invalid-name
 
-    def __make_update_trie_factory(update):
+    def __make_update_trie_factory(update):  # pylint: disable=unused-private-member
         def factory(trie_ctor, d):
             t = trie_ctor()
             update(t, d)
             return t
         return factory
 
-    def __setter_trie_factory(trie_ctor, d):
+    def __setter_trie_factory(trie_ctor, d):  # pylint: disable=unused-private-member
         t = trie_ctor()
         for k, v in d.items():
             t[k] = v
         return t
 
-    def __sorted_trie_factory(trie_ctor, d):
+    def __sorted_trie_factory(trie_ctor, d):  # pylint: disable=unused-private-member
         t = trie_ctor(d)
         t.enable_sorting(True)
         return t
@@ -54,13 +54,13 @@ class _TrieFactoryParameteriser(object):
         lambda trie_ctor, d: trie_ctor(trie_ctor(d))
     ), (
         'UpdateWithNamedArgs',
-        __make_update_trie_factory(lambda t, d: t.update(**d))
+        __make_update_trie_factory(lambda t, d: t.update(**d))  # pylint: disable=too-many-function-args
     ), (
         'UpdateWithTuples',
-        __make_update_trie_factory(lambda t, d: t.update(d.items()))
+        __make_update_trie_factory(lambda t, d: t.update(d.items()))  # pylint: disable=too-many-function-args
     ), (
         'UpdateWithDict',
-        __make_update_trie_factory(lambda t, d: t.update(d))
+        __make_update_trie_factory(lambda t, d: t.update(d))  # pylint: disable=too-many-function-args
     ), (
         'Setters',
         __setter_trie_factory
@@ -488,10 +488,10 @@ class TrieTestCase(unittest.TestCase):
                            self._OTHER_KEY), 42)
         t = trie_factory(self._TRIE_CTOR, d)
 
-        pickled = pickle.dumps(t)
-        u = pickle.loads(pickled)
-
-        self.assertEqual(t, u)
+        for protocol in range(0, pickle.HIGHEST_PROTOCOL + 1):
+            pickled = pickle.dumps(t, protocol=protocol)
+            u = pickle.loads(pickled)
+            self.assertEqual(t, u, 'protocol=' + str(protocol))
 
     def _test_copy_impl(self, make_copy, is_shallow):
         key = self._LONG_KEY
@@ -751,13 +751,13 @@ class StringTrieTestCase(TrieTestCase):
     _PICKLED_PROTO_0 = (
         'Y2NvcHlfcmVnCl9yZWNvbnN0cnVjdG9yCnAwCihjcHlndHJpZQpTdHJpbmdUcmllCnAxCm'
         'NfX2J1aWx0aW5fXwpvYmplY3QKcDIKTnRwMwpScDQKKGRwNQpWX3NlcGFyYXRvcgpwNgpW'
-        'LwpwNwpzVl9yb290CnA4CmcwCihjcHlndHJpZQpfTm9kZQpwOQpnMgpOdHAxMApScDExCi'
-        'hscDEyCkwxTAphVmZvbwpwMTMKYUw0MkwKYUwtMUwKYUwxTAphVmJhcgpwMTQKYUw0MkwK'
-        'YUwtMUwKYUwxTAphVmJhegpwMTUKYUw0MkwKYWJzVl9zb3J0ZWQKcDE2CkkwMApzYi4=')
+        'fgpwNwpzVl9yb290CnA4CmcwCihjcHlndHJpZQpfTm9kZQpwOQpnMgpOdHAxMApScDExCi'
+        'hscDEyCkkxCmFWZm9vCnAxMwphSTQyCmFJLTEKYUkxCmFWYmFyCnAxNAphSTQyCmFJLTEK'
+        'YUkxCmFWYmF6CnAxNQphSTQyCmFic1Zfc29ydGVkCnAxNgpJMDAKc2Iu')
 
     _PICKLED_PROTO_3 = (
         'gANjcHlndHJpZQpTdHJpbmdUcmllCnEAKYFxAX1xAihYCgAAAF9zZXBhcmF0b3JxA1gBAA'
-        'AAL3EEWAUAAABfcm9vdHEFY3B5Z3RyaWUKX05vZGUKcQYpgXEHXXEIKEsBWAMAAABmb29x'
+        'AAfnEEWAUAAABfcm9vdHEFY3B5Z3RyaWUKX05vZGUKcQYpgXEHXXEIKEsBWAMAAABmb29x'
         'CUsqSv////9LAVgDAAAAYmFycQpLKkr/////SwFYAwAAAGJhenELSyplYlgHAAAAX3Nvcn'
         'RlZHEMiXViLg==')
 
@@ -916,9 +916,8 @@ class TraverseTest(unittest.TestCase):
             if value is self._SENTINEL and len(children) == 1:
                 # There is only one prefix.
                 return children[0]
-            else:
-                return self._TestNode(
-                    path_conv(path), bool(children), children, value)
+            return self._TestNode(
+                path_conv(path), bool(children), children, value)
 
         r = t.traverse(make)
         # Result:
@@ -950,10 +949,9 @@ class TraverseTest(unittest.TestCase):
             cnt[0] += 1
             if path and path[0] == 'a':
                 return None
-            else:
-                children = [ch for ch in children if ch is not None]
-                return self._TestNode(
-                    path_conv(path), bool(children), children, value)
+            children = [ch for ch in children if ch is not None]
+            return self._TestNode(
+                path_conv(path), bool(children), children, value)
 
         r = t.traverse(make)
         # Result:
@@ -1051,5 +1049,117 @@ class RecursionTest(unittest.TestCase):
         #self.assertEqual((100 * 1000 - (100 * 99) / 2) * 2 + 1, len(nodes))
 
 
+class EqualityTest(unittest.TestCase):
+    """Tests for equality comparisons."""
+
+    def test_trie_ne_char_trie(self):
+        """Check that Trie and CharTrie are ne even if they have the same nodes.
+
+        Trie returns keys as tuples while CharTrie returns them as strings.
+        Because of that, even if they have the same nodes, they will produce
+        different (key, value) pairs.
+        """
+        trie = pygtrie.Trie([('foo', 42)])
+        char_trie = pygtrie.CharTrie([('foo', 42)])
+        # pylint: disable=protected-access
+        self.assertTrue(trie._root.equals(char_trie._root))
+        self.assertNotEqual(trie, char_trie)
+        self.assertFalse(trie.strictly_equals(char_trie))
+
+    def test_strieng_trie_eq(self):
+        """Test that StringTrie takes separator into consideration."""
+        trie_slash = pygtrie.StringTrie([('foo/bar', 42)])
+        trie_dot0 = pygtrie.StringTrie([('foo.bar', 42)], separator='.')
+        trie_dot1 = pygtrie.StringTrie([('foo.bar', 42)], separator='.')
+        self.assertEqual(trie_slash, trie_slash)
+        self.assertTrue(trie_slash.strictly_equals(trie_slash))
+        # pylint: disable=protected-access
+        self.assertTrue(trie_slash._root.equals(trie_dot0._root))
+        self.assertNotEqual(trie_slash, trie_dot0)
+        self.assertFalse(trie_slash.strictly_equals(trie_dot0))
+        self.assertEqual(trie_dot0, trie_dot1)
+        self.assertTrue(trie_dot0.strictly_equals(trie_dot1))
+
+    def test_mapping_eq(self):
+        """Test comparison with non-Trie mapping types."""
+        # pylint: disable=import-outside-toplevel
+        try:
+            from collections import abc
+        except ImportError:  # Python 2 compatibility
+            abc = collections
+
+        class Mapping(abc.Mapping):
+            def __getitem__(self, key):
+                if key == 'foo/bar.baz':
+                    return 42
+                raise KeyError(key)
+
+            def __iter__(self):
+                return iter(('foo/bar.baz',))
+
+            def __len__(self):
+                return 1
+
+        trie_slash = pygtrie.StringTrie([('foo/bar.baz', 42)], separator='/')
+        trie_dot = pygtrie.StringTrie([('foo/bar.baz', 42)], separator='.')
+        char_trie = pygtrie.CharTrie([('foo/bar.baz', 42)])
+        dictionary = {'foo/bar.baz': 42}
+        mapping = Mapping()
+
+        all_tries = (trie_slash, trie_dot, char_trie)
+        for a in all_tries:
+            for b in all_tries:
+                self.assertEqual(a is b, a.strictly_equals(b))
+
+        all_mappings = (trie_slash, trie_dot, char_trie, dictionary, mapping)
+        for a in all_mappings:
+            for b in all_mappings:
+                self.assertEqual(a, b)
+
+        dictionary['other-key'] = 24
+        all_tries = (trie_slash, trie_dot, char_trie)
+        for trie in all_tries:
+            self.assertNotEqual(dictionary, trie)
+            self.assertNotEqual(trie, dictionary)
+
+
+class MergeTest(unittest.TestCase):
+    """Tests for merge method."""
+
+    def test_merge_tries(self):
+        trie = pygtrie.Trie({'foo': 1, 'bar': 2})
+
+        def test(want, src, overwrite=False):
+            trie.merge(src, overwrite=overwrite)
+            self.assertEqual(0, len(src))
+            self.assertEqual(pygtrie.Trie(want), trie)
+
+        test({'foo': 1, 'bar': 2, 'baz': 3},
+             pygtrie.Trie({'bar': 0, 'baz': 3}))
+        test({'foo': 1, 'bar': 2, 'baz': 4, 'fo': 5},
+             pygtrie.Trie({'baz': 4, 'fo': 5}), overwrite=True)
+        test({'foo': 1, 'bar': 2, 'baz': 4, 'fo': 5, 'qux': 6},
+             pygtrie.CharTrie({'qux': 6}))
+
+        st = pygtrie.StringTrie({'foo/bar/baz': 42})
+        self.assertRaises(TypeError, trie.merge, st)
+
+    def test_merge_string_tries(self):
+
+        def test(want, other, overwrite):
+            trie = pygtrie.StringTrie({'foo/bar': 42})
+            trie.merge(other, overwrite=overwrite)
+            self.assertEqual(want, dict(trie.items()))
+
+        test({'foo/bar': 42, 'bar/baz': 2},
+             pygtrie.StringTrie({'foo.bar': 4, 'bar.baz': 2}, separator='.'),
+             False)
+        test({'foo/bar': 4, 'bar/baz': 2},
+             pygtrie.StringTrie({'foo.bar': 4, 'bar.baz': 2}, separator='.'),
+             True)
+        test({'foo/bar': 42, 'q/u/x': 2}, pygtrie.Trie({'qux': 2}), False)
+        test({'foo/bar': 42, 'q/u/x': 2}, pygtrie.CharTrie({'qux': 2}), False)
+
+
 if __name__ == '__main__':
     unittest.main()
